//==============================================================================
#define F_CPU 8000000UL

#include <avr/io.h>
#include <avr/interrupt.h>   
#include <avr/sleep.h>
#include <util/delay.h>

void initTM1637(void);

#define LEDBLUE PB5   // синий светодиод
extern unsigned char chas, mins, sec;

// режим работы, 1 - питание от сети, 0 - работа от аккумулятора
extern unsigned char regim;  // текущее значение

//==============================================================================
// настройка таймера для работы в асинхронном режиме с кварцем 32768 Гц
//------------------------------------------------------------------------------
void initT2(void)
{ ASSR |= (1<<AS2);           // Включаем асинхронный режим T2
  TCNT2 = 0;
  TCCR2A = 0x00;
  TCCR2B = (5<<CS20);
  OCR2A  = 0x00;
  OCR2B  = 0x00;
  TIMSK2 |= (1 << TOIE2);
}
//==============================================================================
void minit(void)
{ CLKPR = 0x80; CLKPR = 0x00; // Crystal Oscillator division factor: 1
  UCSR0B = 0x00;  // USART initialization - disabled
// Analog Comparator initialization - Off
// Input Capture by Timer/Counter 1: Off
  ACSR   = 0x80;
  ADCSRB = 0x00;
  DIDR1  = 0x00;
  ADCSRA = 0x00;  // ADC initialization - disabled
  SPCR   = 0x00;  // SPI initialization - disabled
  TWCR   = 0x00;  // TWI initialization - disabled
// External Interrupt(s) initialization - INT0=Off INT1=Off
  EICRA = 0x00;
  EIMSK = 0x00;
  PCICR = 0x00;
  TIMSK1 = 0x00;  // Timer/Counter 1 Interrupt(s) initialization
  TIMSK2 = 0x00;  // Timer/Counter 2 Interrupt(s) initialization

// шина данных для дисплея, выходы 8,9,2,3,4,5,6,7 arduinы
  PORTB &= 0b11111100;  DDRB |= 0b00000011;
  PORTD &= 0b00000011;  DDRD |= 0b11111100;
  PORTC &= 0b00000000;  DDRC |= 0b00011111; // шина управления

  initT2();
  initTM1637();
  
  set_sleep_mode(SLEEP_MODE_PWR_SAVE);  // установка параметров малопотребляющего режима
  sleep_enable();

  DDRB |= (1<<LEDBLUE);
  
  sei();
}
//==============================================================================
// "спящий" режим
void mslipping(void)
{ PORTB = 0b11000000;
  PORTD &= 0b00000000;
  PORTC &= 0b00000000;
  _delay_ms(1);
  while(ASSR & (1<<TCN2UB)) {;}
  _delay_us(100);
  asm("sleep");
  _delay_us(100);
}
//==============================================================================
ISR(TIMER2_OVF_vect)
{ sec++;
  if (sec > 59)
  { sec = 0;
    mins++; 
    if (mins > 59)
    { mins = 0;
      chas++;
      if (chas > 23) chas = 0;
    }
  }
  if (regim == 0)
  { 
    PORTB |= (1<<LEDBLUE);
    _delay_us(500);
    PORTB &= ~(1<<LEDBLUE);
  }
}
//==============================================================================

