//==============================================================================
// библиотека для TM1637
//==============================================================================
#define F_CPU 8000000UL

#include <avr/io.h>
#include <util/delay.h>

#define CLK     1
#define DIO     0
#define PORT    PORTD
#define DDR     DDRD
#define PIN     PIND

unsigned char YARKO = 0; // настройка яркости (0...7(

unsigned char symbol[19] = // знакогенератор
{ 0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F, // 0-9
  0x77,0x7C,0x39,0x5E,0x79,0x71,  // A,b,C,d,E,F
  0b01110100, // h
  0x40,0x00 // "-"," "
};
//==============================================================================
void TM1637start(void)
{ PORT |= (1<<CLK);
  _delay_us(2);
  PORT |= (1<<DIO);
  _delay_us(2);
  PORT &= ~(1<<DIO);
}
//==============================================================================
void TM1637stop(void)
{ PORT &= ~((1<<CLK)|(1<<DIO));
}
//==============================================================================
void TM1637write(unsigned char data)
{   unsigned char i;
  unsigned char tr[8] = 
    { 0b00000001,
      0b00000010,
      0b00000100,
      0b00001000,
      0b00010000,
      0b00100000,
      0b01000000,
      0b10000000
    };
    for(i = 0; i < 8; i++)        
    { PORT &= ~(1<<CLK);
      if(data & tr[i]) PORT |= (1<<DIO);
      else             PORT &= ~(1<<DIO);
    _delay_us(3);
      PORT |= (1<<CLK);
    _delay_us(3);  
    }  
    PORT &= ~(1<<CLK);
    _delay_us(5);

    DDR &= ~(1<<DIO);
    while((PIN & (1<<DIO))); 
    DDR |= (1<<DIO);

    PORT |= (1<<CLK);
    _delay_us(2);
    PORT &= ~(1<<CLK);  
}
//==============================================================================
void TM1637display(unsigned char Pos,unsigned char data)
{ TM1637start();
  TM1637write(0x44);
  TM1637stop();

  TM1637start();
  TM1637write(Pos|0xC0);
  TM1637write(data);
  TM1637stop();

  TM1637start();
  TM1637write(0x88 + YARKO);  // яркость
  TM1637stop();
}
//==============================================================================
// выводит температуру и давление
void TM1637tislo(
  unsigned int tislo1,  // температура 33
  unsigned int tislo2)  // давление 777
{ unsigned char t11 = tislo1 / 10, t12 = tislo1 % 10;
//------------------------------------------
  unsigned char t21 = 0,t22 = 0,t23 = 0;
  t23 = tislo2 % 10;  tislo2 = tislo2 / 10;
  t22 = tislo2 % 10;  tislo2 = tislo2 / 10;
  t21 = tislo2;
//------------------------------------------
  TM1637display(0,symbol[t11]);
  TM1637display(1,symbol[t12]);
  TM1637display(2,0);
  TM1637display(3,symbol[t21]);
  TM1637display(4,symbol[t22]);
  TM1637display(5,symbol[t23]);
}
//==============================================================================
// выводит время в формате HH.MM.SS
void TM1637time(unsigned char chas,unsigned char mins,unsigned char sec)
{ TM1637display(0,symbol[chas/10]);
  TM1637display(1,symbol[chas%10] + 128); // + точка
  TM1637display(2,symbol[mins/10]);
  TM1637display(3,symbol[mins%10] + 128); // + точка
  TM1637display(4,symbol[sec/10]);
  TM1637display(5,symbol[sec%10]);
}
//==============================================================================
void initTM1637(void)
{ DDR |= (1<<CLK)|(1<<DIO);
}
//==============================================================================
void i2cAsk(void)
{   PORT &= ~(1<<CLK);
    _delay_us(5);
    DDR &= ~(1<<DIO);
    while((PIN & (1<<DIO))); 
    DDR |= (1<<DIO);
    PORT |= 1<<CLK;
    _delay_us(2);
    PORT &= ~(1<<CLK);  
}
//==============================================================================
// читаем кнопки
//------------------------------------------------------------------------------
unsigned char TM1637knop(void)
{ unsigned char retval = 0, i;
  unsigned char tr[8] = 
  { 0b10000000,
    0b01000000,
    0b00100000,
    0b00010000,
    0b00001000,
    0b00000100,
    0b00000010,
    0b00000001
  };  

  TM1637start();
  TM1637write(0x42);

  DDR &= ~(1<<DIO);
  for(i=0; i < 8; i++)
  { PORT &= ~(1<<CLK);
//    retval = retval>>1;
    _delay_us(30);
      PORT |= (1<<CLK);
    if ((PIN & (1<<DIO)) != 0)
    { retval = retval + tr[i];
    }
    _delay_us(30);
  }
  i2cAsk();
  TM1637stop();
  return retval;
}
//==============================================================================

